@@ -62,4 +62,15 @@ promise_value <- function(name) {
   .Call(rlang_promise_value, name, caller_env())
 }
 
+warningcall <- function(call, msg) {
+  .Call(rlang_test_Rf_warningcall, call, msg)
+}
+errorcall <- function(call, msg) {
+  .Call(rlang_test_Rf_errorcall, call, msg)
+}
+
+sexp_attrib <- function(x) {
+  .Call(rlang_attrib, x)
+}
+
 # nocov end
