@@ -102,6 +102,9 @@ trace_find_bottom <- function(bottom, frames) {
   if (is_environment(bottom)) {
     top <- detect_index(frames, is_reference, bottom)
     if (!top) {
+      if (is_reference(bottom, global_env())) {
+        return(int())
+      }
       abort("Can't find `bottom` on the call tree")
     }
 
@@ -220,28 +223,55 @@ normalise_parents <- function(parents) {
   parents
 }
 
-new_trace <- function(calls, parents, envs) {
-  stopifnot(is.list(calls), is.integer(parents), length(calls) == length(parents))
+new_trace <- function(calls, parents, envs, indices = NULL) {
+  indices <- indices %||% seq_along(calls)
+
+  n <- length(calls)
+  stopifnot(
+    is_list(calls),
+    is_integer(parents, n),
+    is_integer(indices, n)
+  )
 
   structure(
     list(
       calls = calls,
       parents = parents,
-      envs = envs
+      envs = envs,
+      indices = indices
     ),
     class = "rlang_trace"
   )
 }
 
+trace_reset_indices <- function(trace) {
+  trace$indices <- seq_len(trace_length(trace))
+  trace
+}
+
 # Methods -----------------------------------------------------------------
 
+# For internal use only
+c.rlang_trace <- function(...) {
+  traces <- list(...)
+
+  calls <- flatten(map(traces, `[[`, "calls"))
+  parents <- flatten_int(map(traces, `[[`, "parents"))
+  envs <- flatten(map(traces, `[[`, "envs"))
+  indices <- flatten_int(map(traces, `[[`, "indices"))
+
+  new_trace(calls, parents, envs, indices)
+}
+
 #' @export
 format.rlang_trace <- function(x,
                                ...,
-                               simplify = c("collapse", "branch", "none"),
+                               simplify = c("none", "collapse", "branch"),
                                max_frames = NULL,
                                dir = getwd(),
                                srcrefs = NULL) {
+  x <- trace_reset_indices(x)
+
   switch(arg_match(simplify),
     none = trace_format(x, max_frames, dir, srcrefs),
     collapse = trace_format_collapse(x, max_frames, dir, srcrefs),
@@ -259,7 +289,7 @@ trace_format <- function(trace, max_frames, dir, srcrefs) {
   }
 
   tree <- trace_as_tree(trace, dir = dir, srcrefs = srcrefs)
-  cli_tree(tree)
+  cli_tree(tree, indices = trace$indices)
 }
 trace_format_collapse <- function(trace, max_frames, dir, srcrefs) {
   trace <- trace_simplify_collapse(trace)
@@ -271,7 +301,7 @@ trace_format_trail <- function(trace, max_frames, dir, srcrefs) {
   tree <- trace_as_tree(trace, dir = dir, srcrefs = srcrefs)
 
   branch <- tree[-1, ][["call"]]
-  cli_branch(branch, max_frames)
+  cli_branch(branch, max = max_frames, indices = trace$indices)
 }
 
 format_collapsed <- function(what, n) {
@@ -291,13 +321,21 @@ format_collapsed_trail <- function(what, n, style = NULL) {
   format_collapsed(what, n)
 }
 
-cli_branch <- function(lines, max = NULL, style = NULL) {
+cli_branch <- function(lines, max = NULL, style = NULL, indices = NULL) {
   if (!length(lines)) {
     return(chr())
   }
 
-  style <- style %||% cli_box_chars()
-  lines <- paste0(" ", style$h, lines)
+  numbered <- length(indices)
+  if (numbered) {
+    indices <- pad_spaces(as.character(indices))
+    indices <- paste0(" ", indices, ". ")
+    padding <- nchar(indices[[1]])
+    lines <- paste0(silver(indices), lines)
+  } else {
+    style <- style %||% cli_box_chars()
+    lines <- paste0(" ", style$h, lines)
+  }
 
   if (is_null(max)) {
     return(lines)
@@ -316,7 +354,11 @@ cli_branch <- function(lines, max = NULL, style = NULL) {
   style <- style %||% cli_box_chars()
   n_collapsed <- n - max
 
-  collapsed_line <- format_collapsed_trail("...", n_collapsed, style = style)
+  if (numbered) {
+    collapsed_line <- paste0(spaces(padding), "...")
+  } else {
+    collapsed_line <- format_collapsed_trail("...", n_collapsed, style = style)
+  }
 
   if (max == 1L) {
     lines <- chr(
@@ -341,7 +383,7 @@ cli_branch <- function(lines, max = NULL, style = NULL) {
 #' @export
 print.rlang_trace <- function(x,
                               ...,
-                              simplify = c("collapse", "branch", "none"),
+                              simplify = c("none", "branch", "collapse"),
                               max_frames = NULL,
                               dir = getwd(),
                               srcrefs = NULL) {
@@ -384,25 +426,16 @@ trace_subset <- function(x, i) {
     i <- setdiff(seq_len(n), abs(i))
   }
 
-  calls <- x$calls[i]
-  envs <- x$envs[i]
   parents <- match(as.character(x$parents[i]), as.character(i), nomatch = 0)
 
-  new_trace(calls, parents, envs)
+  new_trace(
+    calls = x$calls[i],
+    parents = parents,
+    envs = x$envs[i],
+    indices = x$indices[i]
+  )
 }
 
-# For internal use only
-c.rlang_trace <- function(...) {
-  traces <- list(...)
-
-  calls <- flatten(map(traces, `[[`, "calls"))
-  parents <- flatten_int(map(traces, `[[`, "parents"))
-  envs <- flatten(map(traces, `[[`, "envs"))
-
-  new_trace(calls, parents, envs)
-}
-
-
 # Subsets sibling nodes, at the level of the rightmost leaf by
 # default. Supports full vector subsetting semantics (negative values,
 # missing index, etc).
@@ -708,13 +741,26 @@ trace_simplify_collapse <- function(trace) {
 
 #' Last `abort()` error
 #'
-#' This returns the last error thrown with [abort()]. The error is
-#' printed with a backtrace.
+#' @description
+#'
+#' * `last_error()` returns the last error thrown with [abort()]. The
+#'   error is printed with a backtrace in simplified form.
+#'
+#' * `last_trace()` is a shortcut to return the backtrace stored in
+#'   the last error. This backtrace is printed in full form.
 #'
 #' @export
 last_error <- function() {
+  if (is_null(last_error_env$cnd)) {
+    abort("Can't show last error because no error was recorded yet")
+  }
   last_error_env$cnd
 }
+#' @rdname last_error
+#' @export
+last_trace <- function() {
+  last_error()$trace
+}
 
 
 # Printing ----------------------------------------------------------------
@@ -748,7 +794,7 @@ trace_as_tree <- function(x, dir = getwd(), srcrefs = NULL) {
 # FIXME: Add something like call_deparse_line()
 trace_call_text <- function(call, collapse) {
   if (is_null(collapse)) {
-    return(label(call))
+    return(as_label(call))
   }
 
   if (is_call(call, "%>%")) {
@@ -757,7 +803,7 @@ trace_call_text <- function(call, collapse) {
     call <- call2(node_car(call), quote(...))
   }
 
-  text <- label(call)
+  text <- as_label(call)
   if (collapse > 0L) {
     n_collapsed_text <- sprintf(" ... +%d", collapse)
   } else {
