@@ -388,7 +388,7 @@ test_that("data pronoun always skips functions", {
 })
 
 test_that("leaked quosure masks are not mistaken with data masks", {
-  scoped_silent_retirement()
+  scoped_lifecycle_silence()
   e <- eval_tidy(quote(current_env()))
   expect_no_error(eval_tidy("foo", e))
 })
@@ -404,6 +404,11 @@ test_that("quosures look for data masks lexically", {
   expect_identical(out, list(mtcars$cyl, mtcars$disp))
 })
 
+test_that("can evaluate quosures created in the data mask without infloop", {
+  quo <- eval_tidy(quote(quo(a)), list(a = "foo"))
+  expect_identical(eval_bare(quo, quo_get_env(quo)), "foo")
+})
+
 
 # Lifecycle ----------------------------------------------------------
 
@@ -416,6 +421,7 @@ test_that("as_data_mask() and new_data_mask() warn once when passed a parent", {
 })
 
 test_that("supplying environment as data is deprecated", {
+  scoped_options(lifecycle_verbose_soft_deprecation = TRUE)
   `_x` <- "foo"
   expect_warning(eval_tidy("foo", current_env()), "deprecated")
   expect_identical(eval_tidy(quo(`_x`), current_env()), "foo")
